@@ -5,14 +5,12 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2, incorporated herein by reference.
 
-import os, glob, shlex, sys, time
+import os, sys, time
 
 from PyQt4.QtCore import *
 from PyQt4.QtGui import *
 from PyQt4.Qsci import QsciScintilla
 
-from mercurial import util
-
 from tortoisehg.util import hglib, paths
 from tortoisehg.hgqt.i18n import _, localgettext
 from tortoisehg.hgqt import qtlib, qscilib, thread
@@ -30,7 +28,7 @@ def stopProgress(topic):
 class ProgressMonitor(QWidget):
     'Progress bar for use in workbench status bar'
     def __init__(self, topic, parent):
-        super(ProgressMonitor, self).__init__(parent=parent)
+        super(ProgressMonitor, self).__init__(parent)
 
         hbox = QHBoxLayout()
         hbox.setContentsMargins(*(0,)*4)
@@ -60,8 +58,10 @@ class ProgressMonitor(QWidget):
         self.idle = True
 
     def setcounts(self, cur, max):
-        self.pbar.setMaximum(max)
-        self.pbar.setValue(cur)
+        # cur and max may exceed INT_MAX, which confuses QProgressBar
+        assert max != 0
+        self.pbar.setMaximum(100)
+        self.pbar.setValue(int(cur * 100 / max))
 
     def unknown(self):
         self.pbar.setMinimum(0)
@@ -72,7 +72,7 @@ class ThgStatusBar(QStatusBar):
     linkActivated = pyqtSignal(QString)
 
     def __init__(self, parent=None):
-        QStatusBar.__init__(self, parent=parent)
+        QStatusBar.__init__(self, parent)
         self.topics = {}
         self.lbl = QLabel()
         self.lbl.linkActivated.connect(self.linkActivated)
@@ -134,6 +134,39 @@ class ThgStatusBar(QStatusBar):
             pm.unknown()
 
 
+def _quotecmdarg(arg):
+    # only for display; no use to construct command string for os.system()
+    if not arg or ' ' in arg or '\\' in arg or '"' in arg:
+        return '"%s"' % arg.replace('"', '\\"')
+    else:
+        return arg
+
+def _prettifycmdline(cmdline):
+    r"""Build pretty command-line string for display
+
+    >>> _prettifycmdline(['--repository', 'foo', 'status'])
+    'status'
+    >>> _prettifycmdline(['--cwd', 'foo', 'resolve', '--', '--repository'])
+    'resolve -- --repository'
+    >>> _prettifycmdline(['log', 'foo\\bar', '', 'foo bar', 'foo"bar'])
+    'log "foo\\bar" "" "foo bar" "foo\\"bar"'
+    """
+    try:
+        argcount = cmdline.index('--')
+    except ValueError:
+        argcount = len(cmdline)
+    printables = []
+    pos = 0
+    while pos < argcount:
+        if cmdline[pos] in ('-R', '--repository', '--cwd'):
+            pos += 2
+        else:
+            printables.append(cmdline[pos])
+            pos += 1
+    printables.extend(cmdline[argcount:])
+
+    return ' '.join(_quotecmdarg(e) for e in printables)
+
 class Core(QObject):
     """Core functionality for running Mercurial command.
     Do not attempt to instantiate and use this directly.
@@ -222,7 +255,7 @@ class Core(QObject):
             if display:
                 cmd = '%% hg %s\n' % display
             else:
-                cmd = '%% hg %s\n' % ' '.join(cmdline)
+                cmd = '%% hg %s\n' % _prettifycmdline(cmdline)
             self.output.emit(cmd, 'control')
             proc.start(exepath, cmdline, QIODevice.ReadOnly)
 
@@ -274,7 +307,8 @@ class Core(QObject):
 
         cmdline = self.queue.pop(0)
 
-        self.thread = thread.CmdThread(cmdline, self.display, self.parent())
+        display = self.display or _prettifycmdline(cmdline)
+        self.thread = thread.CmdThread(cmdline, display, self.parent())
         self.thread.started.connect(self.onCommandStarted)
         self.thread.commandFinished.connect(self.onThreadFinished)
 
@@ -317,11 +351,13 @@ class Core(QObject):
             self.stbar.showMessage(status, error)
 
         self.display = None
+        self.thread.setParent(None)  # assist gc
         if ret == 0 and self.runNext():
             return # run next command
         else:
             self.queue = []
             text = self.thread.rawoutput.join('')
+            self.thread = None
             self.rawoutlines = [hglib.fromunicode(text, 'replace')]
 
         self.commandFinished.emit(ret)
@@ -338,6 +374,7 @@ class LogWidget(QsciScintilla):
         self.setWrapMode(QsciScintilla.WrapCharacter)
         self._initfont()
         self._initmarkers()
+        qscilib.unbindConflictedKeys(self)
 
     def _initfont(self):
         tf = qtlib.getfont('fontoutputlog')
@@ -375,292 +412,6 @@ class LogWidget(QsciScintilla):
         return iter(self._markers[l] for l in str(label).split()
                     if l in self._markers)
 
-class _LogWidgetForConsole(LogWidget):
-    """Wrapped LogWidget for ConsoleWidget"""
-
-    returnPressed = pyqtSignal(unicode)
-    """Return key pressed when cursor is on prompt line"""
-
-    _prompt = '% '
-
-    def __init__(self, parent=None):
-        super(_LogWidgetForConsole, self).__init__(parent)
-        self._prompt_marker = self.markerDefine(QsciScintilla.Background)
-        self.setMarkerBackgroundColor(QColor('#e8f3fe'), self._prompt_marker)
-        self.cursorPositionChanged.connect(self._updatePrompt)
-
-    def keyPressEvent(self, event):
-        if event.key() in (Qt.Key_Return, Qt.Key_Enter):
-            if self._cursoronpromptline():
-                self.returnPressed.emit(self.commandText())
-            return
-        super(_LogWidgetForConsole, self).keyPressEvent(event)
-
-    def setPrompt(self, text):
-        if text == self._prompt:
-            return
-        self.clearPrompt()
-        self._prompt = text
-        self.openPrompt()
-
-    @pyqtSlot()
-    def openPrompt(self):
-        """Show prompt line and enable user input"""
-        self.closePrompt()
-        self.markerAdd(self.lines() - 1, self._prompt_marker)
-        self.append(self._prompt)
-        self.setCursorPosition(self.lines() - 1, len(self._prompt))
-        self.setReadOnly(False)
-
-        # make sure the prompt line is visible. Because QsciScintilla may
-        # delay line wrapping, setCursorPosition() doesn't always scrolls
-        # to the correct position.
-        # http://www.scintilla.org/ScintillaDoc.html#LineWrapping
-        self.SCN_PAINTED.connect(self._scrollCaretOnPainted)
-
-    @pyqtSlot()
-    def _scrollCaretOnPainted(self):
-        self.SCN_PAINTED.disconnect(self._scrollCaretOnPainted)
-        self.SendScintilla(self.SCI_SCROLLCARET)
-
-    @pyqtSlot()
-    def closePrompt(self):
-        """Disable user input"""
-        if self.commandText():
-            self._setmarker((self.lines() - 1,), 'control')
-        self.markerDelete(self.lines() - 1, self._prompt_marker)
-        self._newline()
-        self.setCursorPosition(self.lines() - 1, 0)
-        self.setReadOnly(True)
-
-    @pyqtSlot()
-    def clearPrompt(self):
-        """Clear prompt line"""
-        line = self.lines() - 1
-        if not (self.markersAtLine(line) & (1 << self._prompt_marker)):
-            return
-        self.markerDelete(line)
-        self.setSelection(line, 0, line, self.lineLength(line))
-        self.removeSelectedText()
-
-    @pyqtSlot(int, int)
-    def _updatePrompt(self, line, pos):
-        """Update availability of user input"""
-        if self.markersAtLine(line) & (1 << self._prompt_marker):
-            self.setReadOnly(False)
-            self._ensurePrompt(line)
-        else:
-            self.setReadOnly(True)
-
-    def _ensurePrompt(self, line):
-        """Insert prompt string if not available"""
-        s = unicode(self.text(line))
-        if s.startswith(self._prompt):
-            return
-        for i, c in enumerate(self._prompt):
-            if s[i:i + 1] != c:
-                self.insertAt(self._prompt[i:], line, i)
-                break
-        self.setCursorPosition(line, self.lineLength(line))
-
-    def commandText(self):
-        """Return the current command text"""
-        l = self.lines() - 1
-        if self.markersAtLine(l) & (1 << self._prompt_marker):
-            return self.text(l)[len(self._prompt):]
-        else:
-            return ''
-
-    def _newline(self):
-        if self.lineLength(self.lines() - 1) > 0:
-            self.append('\n')
-
-    def _cursoronpromptline(self):
-        line = self.getCursorPosition()[0]
-        return self.markersAtLine(line) & (1 << self._prompt_marker)
-
-class _ConsoleCmdTable(dict):
-    """Command table for ConsoleWidget"""
-    _cmdfuncprefix = '_cmd_'
-
-    def __call__(self, func):
-        if not func.__name__.startswith(self._cmdfuncprefix):
-            raise ValueError('bad command function name %s' % func.__name__)
-        self[func.__name__[len(self._cmdfuncprefix):]] = func
-        return func
-
-class ConsoleWidget(QWidget):
-    """Console to run hg/thg command and show output"""
-    closeRequested = pyqtSignal()
-
-    progressReceived = pyqtSignal(QString, object, QString, QString,
-                                  object, object)
-    """Emitted when progress received
-
-    Args: topic, pos, item, unit, total, reporoot
-    """
-
-    _cmdtable = _ConsoleCmdTable()
-
-    # TODO: command history and completion
-
-    def __init__(self, parent=None):
-        super(ConsoleWidget, self).__init__(parent)
-        self.setLayout(QVBoxLayout())
-        self.layout().setContentsMargins(0, 0, 0, 0)
-        self._initlogwidget()
-        self.setFocusProxy(self._logwidget)
-        self.setRepository(None)
-        self.openPrompt()
-        self.suppressPrompt = False
-
-    def _initlogwidget(self):
-        self._logwidget = _LogWidgetForConsole(self)
-        self._logwidget.returnPressed.connect(self._runcommand)
-        self.layout().addWidget(self._logwidget)
-
-        # compatibility methods with LogWidget
-        for name in ('openPrompt', 'closePrompt', 'clear'):
-            setattr(self, name, getattr(self._logwidget, name))
-
-    @util.propertycache
-    def _cmdcore(self):
-        cmdcore = Core(False, self)
-        cmdcore.output.connect(self._logwidget.appendLog)
-        cmdcore.commandStarted.connect(self.closePrompt)
-        cmdcore.commandFinished.connect(self.openPrompt)
-        cmdcore.progress.connect(self._emitProgress)
-        return cmdcore
-
-    @util.propertycache
-    def _extproc(self):
-        extproc = QProcess(self)
-        extproc.started.connect(self.closePrompt)
-        extproc.finished.connect(self.openPrompt)
-
-        def handleerror(error):
-            msgmap = {
-                QProcess.FailedToStart: _('failed to run command\n'),
-                QProcess.Crashed: _('crashed\n')}
-            if extproc.state() == QProcess.NotRunning:
-                self._logwidget.closePrompt()
-            self._logwidget.appendLog(
-                msgmap.get(error, _('error while running command\n')),
-                'ui.error')
-            if extproc.state() == QProcess.NotRunning:
-                self._logwidget.openPrompt()
-        extproc.error.connect(handleerror)
-
-        def put(bytes, label=None):
-            self._logwidget.appendLog(hglib.tounicode(bytes.data()), label)
-        extproc.readyReadStandardOutput.connect(
-            lambda: put(extproc.readAllStandardOutput()))
-        extproc.readyReadStandardError.connect(
-            lambda: put(extproc.readAllStandardError(), 'ui.error'))
-
-        return extproc
-
-    @pyqtSlot(unicode, str)
-    def appendLog(self, msg, label):
-        """Append log text from another cmdui"""
-        self._logwidget.clearPrompt()
-        try:
-            self._logwidget.appendLog(msg, label)
-        finally:
-            if not self.suppressPrompt:
-                self.openPrompt()
-
-    @pyqtSlot(object)
-    def setRepository(self, repo):
-        """Change the current working repository"""
-        self._repo = repo
-        self._logwidget.setPrompt('%s%% ' % (repo and repo.displayname or ''))
-
-    @property
-    def cwd(self):
-        """Return the current working directory"""
-        return self._repo and self._repo.root or os.getcwd()
-
-    @pyqtSlot(unicode, object, unicode, unicode, object)
-    def _emitProgress(self, *args):
-        self.progressReceived.emit(
-            *(args + (self._repo and self._repo.root or None,)))
-
-    @pyqtSlot(unicode)
-    def _runcommand(self, cmdline):
-        try:
-            args = list(self._parsecmdline(cmdline))
-        except ValueError, e:
-            self.closePrompt()
-            self._logwidget.appendLog(unicode(e) + '\n', 'ui.error')
-            self.openPrompt()
-            return
-        if not args:
-            self.openPrompt()
-            return
-        cmd = args.pop(0)
-        try:
-            self._cmdtable[cmd](self, args)
-        except KeyError:
-            return self._runextcommand(cmdline)
-
-    def _parsecmdline(self, cmdline):
-        """Split command line string to imitate a unix shell"""
-        try:
-            args = shlex.split(hglib.fromunicode(cmdline))
-        except ValueError, e:
-            raise ValueError(_('command parse error: %s') % e)
-        for e in args:
-            e = util.expandpath(e)
-            if util.any(c in e for c in '*?[]'):
-                expanded = glob.glob(os.path.join(self.cwd, e))
-                if not expanded:
-                    raise ValueError(_('no matches found: %s')
-                                     % hglib.tounicode(e))
-                for p in expanded:
-                    yield p
-            else:
-                yield e
-
-    def _runextcommand(self, cmdline):
-        self._extproc.setWorkingDirectory(hglib.tounicode(self.cwd))
-        self._extproc.start(cmdline, QIODevice.ReadOnly)
-
-    @_cmdtable
-    def _cmd_hg(self, args):
-        self.closePrompt()
-        if self._repo:
-            args = ['--cwd', self._repo.root] + args
-        self._cmdcore.run(args)
-
-    @_cmdtable
-    def _cmd_thg(self, args):
-        from tortoisehg.hgqt import run
-        self.closePrompt()
-        try:
-            if self._repo:
-                args = ['-R', self._repo.root] + args
-            # TODO: show errors
-            run.dispatch(args)
-        finally:
-            self.openPrompt()
-
-    @_cmdtable
-    def _cmd_clear(self, args):
-        self.clear()
-        self.openPrompt()
-
-    @_cmdtable
-    def _cmd_cls(self, args):
-        self.clear()
-        self.openPrompt()
-
-    @_cmdtable
-    def _cmd_exit(self, args):
-        self.clear()
-        self.openPrompt()
-        self.closeRequested.emit()
-
 
 class Widget(QWidget):
     """An embeddable widget for running Mercurial command"""
