@@ -11,7 +11,7 @@ import tempfile
 
 from mercurial import ui, util, error, scmutil, phases
 
-from tortoisehg.util import hglib, shlib, wconfig
+from tortoisehg.util import hglib, shlib, wconfig, hgversion
 
 from tortoisehg.hgqt.i18n import _
 from tortoisehg.hgqt.messageentry import MessageEntry
@@ -67,6 +67,8 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
     commitButtonEnable = pyqtSignal(bool)
     linkActivated = pyqtSignal(QString)
     showMessage = pyqtSignal(unicode)
+    grepRequested = pyqtSignal(unicode, dict)
+    runCustomCommandRequested = pyqtSignal(str, list)
     commitComplete = pyqtSignal()
 
     progress = pyqtSignal(QString, object, QString, QString, object)
@@ -78,9 +80,9 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
     def __init__(self, repo, pats, opts, embedded=False, parent=None, rev=None):
         QWidget.__init__(self, parent)
 
-        repo.configChanged.connect(self.configChanged)
+        repo.configChanged.connect(self.refresh)
         repo.repositoryChanged.connect(self.repositoryChanged)
-        repo.workingBranchChanged.connect(self.workingBranchChanged)
+        repo.workingBranchChanged.connect(self.refresh)
         self.repo = repo
         self._rev = rev
         self.lastAction = None
@@ -95,6 +97,9 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
         self.stwidget.progress.connect(self.progress)
         self.stwidget.linkActivated.connect(self.linkActivated)
         self.stwidget.fileDisplayed.connect(self.fileDisplayed)
+        self.stwidget.grepRequested.connect(self.grepRequested)
+        self.stwidget.runCustomCommandRequested.connect(
+            self.runCustomCommandRequested)
         self.msghistory = []
         self.runner = cmdui.Runner(not embedded, self)
         self.runner.setTitle(_('Commit', 'window title'))
@@ -283,10 +288,14 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
             if ispatch(r):
                 return False
             ctx = r.changectx('.')
-            return not ctx.children() \
-                and ctx.phase() != phases.public \
-                and len(ctx.parents()) < 2 \
-                and len(r.changectx(None).parents()) < 2
+            canamendctx = (ctx.phase() != phases.public) \
+                and len(r.changectx(None).parents()) < 2 \
+                and not ctx.children()
+            # hg < 2.6
+            if hgversion.hgversion < '2.6':
+                canamendctx = canamendctx \
+                    and len(ctx.parents()) < 2
+            return canamendctx
 
         acts = [
             ('commit', _('Commit changes'), _('Commit'), notpatch),
@@ -500,6 +509,9 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
             # do not search for tokens if auto completion is disabled
             # pygments has several infinite loop problems we'd like to avoid
             return
+        if self.msgte.lexer() is None:
+            # qsci will crash if None is passed to QsciAPIs constructor
+            return
         wfile = unicode(wfile)
         self._apis = QsciAPIs(self.msgte.lexer())
         tokens = set()
@@ -566,22 +578,12 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
             self.opts.update(dlg.outopts)
             self.refresh()
 
-    @pyqtSlot()
-    def workingBranchChanged(self):
-        'Repository has detected a change in .hg/branch'
-        self.refresh()
-
     @pyqtSlot()
     def repositoryChanged(self):
         'Repository has detected a changelog / dirstate change'
         self.refresh()
         self.stwidget.refreshWctx() # Trigger reload of working context
 
-    @pyqtSlot()
-    def configChanged(self):
-        'Repository is reporting its config files have changed'
-        self.refresh()
-
     @pyqtSlot()
     def refreshWctx(self):
         'User has requested a working context refresh'
@@ -594,6 +596,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
         self.refresh()
         self.stwidget.refreshWctx() # Trigger reload of working context
 
+    @pyqtSlot()
     def refresh(self):
         ispatch = self.repo.changectx('.').thgmqappliedpatch()
         if not self.hasmqbutton:
@@ -614,7 +617,7 @@ class CommitWidget(QWidget, qtlib.TaskWidget):
         opts = commitopts2str(self.opts)
         self.optionslabelfmt = _('<b>Selected Options:</b> %s')
         self.optionslabel.setText(self.optionslabelfmt
-                                  % hglib.tounicode(opts))
+                                  % Qt.escape(hglib.tounicode(opts)))
         self.optionslabel.setVisible(bool(opts))
 
         # Update parent csinfo widget
@@ -1358,26 +1361,13 @@ class CommitDialog(QDialog):
         self.commit.msgte.setFocus()
         qtlib.newshortcutsforstdkey(QKeySequence.Refresh, self, self.refresh)
 
-    def done(self, ret):
-        self.commit.repo.configChanged.disconnect(self.commit.configChanged)
-        self.commit.repo.repositoryChanged.disconnect(self.commit.repositoryChanged)
-        self.commit.repo.workingBranchChanged.disconnect(self.commit.workingBranchChanged)
-        self.commit.repo.repositoryChanged.disconnect(self.updateUndo)
-        super(CommitDialog, self).done(ret)
-
     def linkActivated(self, link):
         link = hglib.fromunicode(link)
         if link.startswith('repo:'):
             from tortoisehg.hgqt.run import qtrun
             qtrun(run, ui.ui(), root=link[len('repo:'):])
-        if link.startswith('shelve:'):
-            repo = self.commit.repo
-            from tortoisehg.hgqt import shelve
-            dlg = shelve.ShelveDialog(repo, self)
-            dlg.finished.connect(dlg.deleteLater)
-            dlg.exec_()
-            self.refresh()
 
+    @pyqtSlot()
     def updateUndo(self):
         BB = QDialogButtonBox
         undomsg = self.commit.canUndo()
