@@ -61,7 +61,7 @@ class HgignoreDialog(QDialog):
         ignorefiles = [repo.wjoin('.hgignore')]
         for name, value in repo.ui.configitems('ui'):
             if name == 'ignore' or name.startswith('ignore.'):
-                ignorefiles.append(os.path.expanduser(value))
+                ignorefiles.append(util.expandpath(value))
 
         filecombo = QComboBox()
         hbox.addWidget(filecombo)
@@ -172,7 +172,7 @@ class HgignoreDialog(QDialog):
         self.contextmenu.exec_(point)
 
     def unknownDoubleClicked(self, item):
-        self.insertFilters([str(item.text())])
+        self.insertFilters([hglib.fromunicode(item.text())])
 
     def insertFilters(self, pats=False, isregexp=False):
         if pats is False:
@@ -248,8 +248,10 @@ class HgignoreDialog(QDialog):
 
         try:
             self.repo.thginvalidate()
+            self.repo.lfstatus = True
             wctx = self.repo[None]
             wctx.status(unknown=True)
+            self.repo.lfstatus = False
         except (EnvironmentError, error.RepoError), e:
             qtlib.WarningMsgBox(_('Unable to read repository status'),
                                 uni(str(e)), parent=self)
