@@ -14,6 +14,8 @@ from PyQt4.Qsci import QsciScintilla, QsciLexerMakefile
 from tortoisehg.hgqt.i18n import _
 from tortoisehg.hgqt import qtlib, qscilib
 
+import re
+
 class MessageEntry(qscilib.Scintilla):
 
     def __init__(self, parent, getCheckedFunc=None):
@@ -29,16 +31,9 @@ class MessageEntry(qscilib.Scintilla):
         self.setAutoIndent(True)
         self.setAutoCompletionSource(QsciScintilla.AcsAPIs)
         self.setAutoCompletionFillupsEnabled(True)
-        self.setLexer(QsciLexerMakefile(self))
-        font = qtlib.getfont('fontcomment').font()
-        self.fontHeight = QFontMetrics(font).height()
-        self.lexer().setFont(font)
-        self.lexer().setColor(QColor(Qt.red), QsciLexerMakefile.Error)
         self.setMatchedBraceBackgroundColor(Qt.yellow)
         self.setIndentationsUseTabs(False)
         self.setBraceMatching(QsciScintilla.SloppyBraceMatch)
-        #self.setIndentationGuidesBackgroundColor(QColor("#e6e6de"))
-        #self.setFolding(QsciScintilla.BoxedFoldStyle)
         # http://www.riverbankcomputing.com/pipermail/qscintilla/2009-February/000461.html
         self.setHorizontalScrollBarPolicy(Qt.ScrollBarAsNeeded)
         self.setVerticalScrollBarPolicy(Qt.ScrollBarAsNeeded)
@@ -48,22 +43,44 @@ class MessageEntry(qscilib.Scintilla):
         self.getChecked = getCheckedFunc
         self.setContextMenuPolicy(Qt.CustomContextMenu)
         self.customContextMenuRequested.connect(self.menuRequested)
+        self.applylexer()
+
+        self._re_boundary = re.compile('[0-9i#]+\.|\(?[0-9i#]+\)|\(@\)')
 
     def setText(self, text):
         result = super(MessageEntry, self).setText(text)
         self.setDefaultEolMode()
         return result
-        
+
+    def applylexer(self):
+        font = qtlib.getfont('fontcomment').font()
+        self.fontHeight = QFontMetrics(font).height()
+        if QSettings().value('msgentry/lexer', True).toBool():
+            self.setLexer(QsciLexerMakefile(self))
+            self.lexer().setColor(QColor(Qt.red), QsciLexerMakefile.Error)
+            self.lexer().setFont(font)
+        else:
+            self.setLexer(None)
+            self.setFont(font)
+
     def menuRequested(self, point):
         line = self.lineAt(point)
         point = self.viewport().mapToGlobal(point)
+        lexerenabled = self.lexer() is not None
 
         def apply():
-            line = 0
+            firstline, firstcol, lastline, lastcol = self.getSelection()
+            if firstline < 0:
+                line = 0
+            else:
+                line = firstline
+            self.beginUndoAction()
             while True:
                 line = self.reflowBlock(line)
-                if line is None:
-                    break;
+                if line is None or (line > lastline > -1):
+                    break
+            self.endUndoAction()
+
         def paste():
             files = self.getChecked()
             self.insert(', '.join(files))
@@ -71,9 +88,17 @@ class MessageEntry(qscilib.Scintilla):
             from tortoisehg.hgqt.settings import SettingsDialog
             dlg = SettingsDialog(True, focus='tortoisehg.summarylen')
             dlg.exec_()
+        def togglelexer():
+            QSettings().setValue('msgentry/lexer', not lexerenabled)
+            self.applylexer()
 
         menu = self.createStandardContextMenu()
         menu.addSeparator()
+        a = menu.addAction(_('Syntax Highlighting'))
+        a.setCheckable(True)
+        a.setChecked(lexerenabled)
+        a.triggered.connect(togglelexer)
+        menu.addSeparator()
         if self.getChecked:
             action = menu.addAction(_('Paste &Filenames'))
             action.triggered.connect(paste)
@@ -99,23 +124,79 @@ class MessageEntry(qscilib.Scintilla):
             return line+1
 
         # find boundaries (empty lines or bounds)
+        def istopboundary(linetext):
+            # top boundary lines are those that begin with a Markdown style marker
+            # or are empty
+            if not linetext:
+                return True
+            if (linetext[0] in '#-*+'):
+                return True
+            if len(linetext) >= 2:
+                if linetext[:2] in ('> ', '| '):
+                    return True
+                if self._re_boundary.match(linetext):
+                    return True
+            return False
+
+        def isbottomboundary(linetext):
+            # bottom boundary lines are those that end with a period
+            # or are empty
+            if not linetext or linetext[-1] == '.':
+                return True
+            return False
+
+        def isanyboundary(linetext):
+            if len(linetext) >= 3:
+                if linetext[:3] in ('~~~', '```', '---', '==='):
+                    return True
+            return False
+
         b = line
         while b and len(lines[b-1]) > 1:
-            b = b - 1
+            linetext = unicode(lines[b].trimmed())
+            if istopboundary(linetext) or isanyboundary(linetext):
+                break
+            if b >= 1:
+                nextlinetext = unicode(lines[b - 1].trimmed())
+                if isbottomboundary(nextlinetext) \
+                        or isanyboundary(nextlinetext):
+                    break
+            b -= 1
+
         e = line
         while e+1 < len(lines) and len(lines[e+1]) > 1:
-            e = e + 1
+            linetext = unicode(lines[e].trimmed())
+            if isbottomboundary(linetext) or isanyboundary(linetext):
+                break
+            nextlinetext =  unicode(lines[e+1].trimmed())
+            if isanyboundary(nextlinetext) or istopboundary(nextlinetext):
+                break
+            e += 1
+
+        if b == e:
+            return line + 1
+
         group = QStringList([lines[l].simplified() for l in xrange(b, e+1)])
+        firstlinetext = unicode(lines[b])
+        if firstlinetext:
+            indentcount = len(firstlinetext) - len(firstlinetext.lstrip())
+            firstindent = firstlinetext[:indentcount]
+        else:
+            indentcount = 0
+            firstindent = ''
         sentence = group.join(' ')
         parts = sentence.split(' ', QString.SkipEmptyParts)
 
         outlines = QStringList()
         line = QStringList()
-        partslen = 0
+        partslen = indentcount - 1
         for part in parts:
             if partslen + len(line) + len(part) + 1 > self.summarylen:
                 if line:
-                    outlines.append(line.join(' '))
+                    linetext = line.join(' ')
+                    if len(outlines) == 0 and firstindent:
+                        linetext = firstindent + linetext
+                    outlines.append(linetext)
                 line, partslen = QStringList(), 0
             line.append(part)
             partslen += len(part)
