@@ -174,7 +174,7 @@ class RepoTreeView(QTreeView):
             else:
                 qtlib.WarningMsgBox(
                     _('Unsupported repository type (%s)') % repotype,
-                    _('Cannot open non mercurial repositories or subrepositories'),
+                    _('Cannot open non Mercurial repositories or subrepositories'),
                     parent=self)
         else:
             # a double-click on non-repo rows opens an editor
@@ -464,7 +464,7 @@ class RepoRegistryView(QDockWidget):
         for name, desc, icon, tip, cb in self._action_defs():
             act = self._actions[name]
             if icon:
-                act.setIcon(qtlib.getmenuicon(icon))
+                act.setIcon(qtlib.geticon(icon))
             if tip:
                 act.setStatusTip(tip)
             if cb:
@@ -647,12 +647,6 @@ class RepoRegistryView(QDockWidget):
                             % root, parent=self)
                 return
 
-            qtlib.WarningMsgBox(
-                _('Failed to add repository'),
-                _('"%s" is not a valid repository inside "%s"') % \
-                (path, root), parent=self)
-            return
-
     def startSettings(self):
         root = self.selitem.internalPointer().rootpath()
         sd = settings.SettingsDialog(configrepo=True, focus='web.name',
@@ -689,7 +683,7 @@ class RepoRegistryView(QDockWidget):
         else:
             qtlib.WarningMsgBox(
                 _('Unsupported repository type (%s)') % repotype,
-                _('Cannot open non mercurial repositories or subrepositories'),
+                _('Cannot open non Mercurial repositories or subrepositories'),
                 parent=self)
 
     def copyPath(self):
