@@ -221,7 +221,7 @@ def _parse(ui, args):
         c.append((o[0], o[1], options[o[1]], o[3]))
 
     try:
-        args = fancyopts.fancyopts(args, c, cmdoptions)
+        args = fancyopts.fancyopts(args, c, cmdoptions, True)
     except fancyopts.getopt.GetoptError, inst:
         raise error.CommandError(cmd, inst)
 
@@ -534,6 +534,7 @@ class _QtRunner(QObject):
             self._mainapp.setOrganizationName('TortoiseHg')
             self._mainapp.setOrganizationDomain('tortoisehg.org')
             self._mainapp.setApplicationVersion(thgversion.version())
+            self._fixlibrarypaths()
             self._installtranslator()
             qtlib.setup_font_substitutions()
             qtlib.fix_application_font()
@@ -573,6 +574,12 @@ class _QtRunner(QObject):
         finally:
             self._mainapp = None
 
+    def _fixlibrarypaths(self):
+        # make sure to use the bundled Qt plugins to avoid ABI incompatibility
+        # http://qt-project.org/doc/qt-4.8/deployment-windows.html#qt-plugins
+        if os.name == 'nt' and getattr(sys, 'frozen', False):
+            self._mainapp.setLibraryPaths([self._mainapp.applicationDirPath()])
+
     def _installtranslator(self):
         if not i18n.language:
             return
@@ -700,10 +707,9 @@ def mq(ui, *pats, **opts):
     from tortoisehg.hgqt.mq import run
     return qtrun(run, ui, *pats, **opts)
 
-def test(ui, *pats, **opts):
-    """test arbitrary widgets"""
-    from tortoisehg.hgqt.mq import run
-    return qtrun(run, ui, *pats, **opts)
+def debugbugreport(ui, *pats, **opts):
+    """open bugreport dialog by exception"""
+    raise Exception(' '.join(pats))
 
 def purge(ui, *pats, **opts):
     """purge unknown and/or ignore files from repository"""
@@ -836,6 +842,11 @@ def thgimport(ui, *pats, **opts):
     from tortoisehg.hgqt.thgimport import run
     return qtrun(run, ui, *pats, **opts)
 
+def revdetails(ui, *pats, **opts):
+    """revision details tool"""
+    from tortoisehg.hgqt.revdetails import run
+    return qtrun(run, ui, *pats, **opts)
+
 ### help management, adapted from mercurial.commands.help_()
 def help_(ui, name=None, with_version=False, **opts):
     """show help for a command, extension, or list of commands
@@ -1115,6 +1126,10 @@ table = {
         (log,
          [('l', 'limit', '', _('(DEPRECATED)'))],
          _('thg log [OPTIONS] [FILE]')),
+    "^revdetails":
+        (revdetails,
+         [('r', 'rev', '', _('the revision to show'))],
+         _('thg revdetails [-r REV]')),
     "manifest":
         (manifest,
          [('r', 'rev', '', _('revision to display')),
@@ -1138,7 +1153,7 @@ table = {
           ('', 'webdir-conf', '',
            _('name of the hgweb config file (DEPRECATED)'))],
          _('thg serve [--web-conf FILE]')),
-    "^sync|synchronize": (sync, [], _('thg sync')),
+    "^sync|synchronize": (sync, [], _('thg sync [PEER]')),
     "^status|st": (status,
          [('c', 'clean', False, _('show files without changes')),
           ('i', 'ignored', False, _('show ignored files'))],
@@ -1167,7 +1182,7 @@ table = {
          _('thg tag [-f] [-l] [-m TEXT] [-r REV] [NAME]')),
     "shelve|unshelve": (shelve, [], _('thg shelve')),
     "rejects": (rejects, [], _('thg rejects [FILE]')),
-    "test": (test, [], _('thg test')),
+    "debugbugreport": (debugbugreport, [], _('thg debugbugreport [TEXT]')),
     "help": (help_, [], _('thg help [COMMAND]')),
     "^purge": (purge, [], _('thg purge')),
     "^qreorder": (qreorder, [], _('thg qreorder')),
