@@ -9,7 +9,8 @@ import os
 
 from mercurial import ui, util, error, extensions, scmutil, phases
 
-from tortoisehg.util import hglib, settings, paths, wconfig, i18n
+from tortoisehg.util import hglib, paths, wconfig, i18n, editor
+from tortoisehg.util import terminal
 from tortoisehg.hgqt.i18n import _
 from tortoisehg.hgqt import qtlib, qscilib, thgrepo, customtools
 
@@ -466,6 +467,11 @@ def genDeferredCombo(opts, func):
     opts['nohist'] = True
     return SettingsCombo(**opts)
 
+def genEditableDeferredCombo(opts, func):
+    'Values retrieved from a function at popup time'
+    opts['canedit'] = True
+    return genDeferredCombo(opts, func)
+
 def genFontEdit(opts):
     return FontEntry(**opts)
 
@@ -499,6 +505,12 @@ def findDiffTools():
 def findMergeTools():
     return hglib.mergetools(ui.ui())
 
+def findEditors():
+    return editor.findeditors(ui.ui())
+
+def findTerminals():
+    return terminal.findterminals(ui.ui())
+
 def genCheckBox(opts):
     opts['nohist'] = True
     return SettingsCheckBox(**opts)
@@ -547,12 +559,14 @@ INFO = (
           'section of your Mercurial configuration files.  If left '
           'unspecified, TortoiseHg will use the selected merge tool. '
           'Failing that it uses the first applicable tool it finds.')),
-    _fi(_('Visual Editor'), 'tortoisehg.editor', genEditCombo,
-        _('Specify the visual editor used to view files.  Format:<br>'
-          'myeditor -flags [$FILE --num=$LINENUM][--search $SEARCH]<br><br>'
-          'See <a href="%s">OpenAtLine</a>'
-          % 'http://bitbucket.org/tortoisehg/thg/wiki/OpenAtLine')),
-    _fi(_('Shell'), 'tortoisehg.shell', genEditCombo,
+    _fi(_('Visual Editor'), 'tortoisehg.editor',
+        (genEditableDeferredCombo, findEditors),
+        _('Specify visual editor, as described in the [editor-tools] '
+          'section of your Mercurial configuration files.  If left '
+          'unspecified, TortoiseHg will use the first applicable tool '
+          'it finds.')),
+    _fi(_('Shell'), 'tortoisehg.shell',
+        (genEditableDeferredCombo, findTerminals),
         _('Specify the command to launch your preferred terminal shell '
           'application. If the value includes the string %(reponame)s, the '
           'name of the repository will be substituted in place of '
@@ -623,6 +637,10 @@ INFO = (
           'the changes are colored green for merge, red for '
           'non-trivial parents, black for normal. '
           'Default: False')),
+    _fi(_('Full Authorname'), 'tortoisehg.fullauthorname', genBoolRBGroup,
+        _('Show full authorname in Logview. If not enabled, '
+          'only a short part, usually name without email is shown. '
+          'Default: False')),
     _fi(_('Task Tabs'), 'tortoisehg.tasktabs', (genDefaultCombo,
          ['east', 'west', 'off']),
         _('Show tabs along the side of the bottom half of each repo '
@@ -673,6 +691,11 @@ INFO = (
         '<li><b>revision</b>: Push the changes in the current branch '
         '<i><u>up to</u> the current revision</i>.</ul><p>'
         'Default: all')),
+    _fi(_('Confirm Push'), 'tortoisehg.confirmpush', genBoolRBGroup,
+        _('Determines if TortoiseHg should show a confirmation dialog '
+          'before pushing changesets. '
+          'If False, push will be performed without any confirmation dialog. '
+          'Default: True')),
     _fi(_('Activate Bookmarks'), 'tortoisehg.activatebookmarks', (genDefaultCombo,
         ['auto', 'prompt', 'never']),
         _('Select when TortoiseHg will show a prompt to activate a bookmark '
