@@ -47,6 +47,8 @@ class StatusWidget(QWidget):
     linkActivated = pyqtSignal(QString)
     showMessage = pyqtSignal(unicode)
     fileDisplayed = pyqtSignal(QString, QString)
+    grepRequested = pyqtSignal(unicode, dict)
+    runCustomCommandRequested = pyqtSignal(str, list)
 
     def __init__(self, repo, pats, opts, parent=None, checkable=True,
                  defcheck='MAR!S'):
@@ -138,20 +140,17 @@ class StatusWidget(QWidget):
         self.filelistToolbar.addSeparator()
         self.filelistToolbar.addWidget(self.refreshBtn)
         self.actions = wctxactions.WctxActions(self.repo, self, checkable)
+        self.actions.runCustomCommandRequested.connect(
+            self.runCustomCommandRequested)
         tv = WctxFileTree(self.repo, checkable=checkable)
         vbox.addLayout(hbox)
         vbox.addWidget(tv)
         split.addWidget(frame)
 
-        if self.pats:
-            def clearPattern():
-                self.pats = []
-                self.refreshWctx()
-                cpb.setVisible(False)
-                self.titleTextChanged.emit(self.getTitle())
-            cpb = QPushButton(_('Remove filter, show root'))
-            vbox.addWidget(cpb)
-            cpb.clicked.connect(clearPattern)
+        self.clearPatternBtn = QPushButton(_('Remove filter, show root'))
+        vbox.addWidget(self.clearPatternBtn)
+        self.clearPatternBtn.clicked.connect(self.clearPattern)
+        self.clearPatternBtn.setVisible(bool(self.pats))
 
         tv.setItemsExpandable(False)
         tv.setRootIsDecorated(False)
@@ -161,13 +160,7 @@ class StatusWidget(QWidget):
         tv.menuRequest.connect(self.onMenuRequest)
         le.textEdited.connect(self.setFilter)
 
-        def statusTypeTrigger(status):
-            status = str(status)
-            for s in statusTypes:
-                val = statusTypes[s]
-                self.opts[val.name] = s in status
-            self.refreshWctx()
-        self.statusfilter.statusChanged.connect(statusTypeTrigger)
+        self.statusfilter.statusChanged.connect(self.setStatusFilter)
 
         self.tv = tv
         self.le = le
@@ -191,6 +184,7 @@ class StatusWidget(QWidget):
         self.fileview.shelveToolExited.connect(self.refreshWctx)
         self.fileview.newChunkList.connect(self.updatePartials)
         self.fileview.chunkSelectionChanged.connect(self.chunkSelectionChanged)
+        self.fileview.grepRequested.connect(self.grepRequested)
         self.fileview.setContext(self.repo[None])
         self.fileview.setMinimumSize(QSize(16, 16))
         vbox.addWidget(self.fileview, 1)
@@ -286,7 +280,7 @@ class StatusWidget(QWidget):
         self.pctx = pctx
 
     @pyqtSlot()
-    def refreshWctx(self, synchronous=False):
+    def refreshWctx(self):
         if self.refthread:
             self.refreshWctxLater.start()
             return
@@ -312,12 +306,9 @@ class StatusWidget(QWidget):
         self.refreshBtn.setEnabled(False)
         self.progress.emit(*cmdui.startProgress(_('Refresh'), _('status')))
         self.refthread = StatusThread(self.repo, self.pctx, self.pats, self.opts)
-        if not synchronous:
-            self.refthread.finished.connect(self.reloadComplete)
+        self.refthread.finished.connect(self.reloadComplete)
         self.refthread.showMessage.connect(self.showMessage)
         self.refthread.start()
-        if synchronous:
-            self.reloadComplete()
 
     @pyqtSlot()
     def reloadComplete(self):
@@ -336,8 +327,11 @@ class StatusWidget(QWidget):
         if match:
             self.setFilter(match)
 
+    def isRefreshingWctx(self):
+        return bool(self.refthread)
+
     def canExit(self):
-        return not self.refthread
+        return not self.isRefreshingWctx()
 
     def updateModel(self, wctx, patchecked):
         self.tv.setSortingEnabled(False)
@@ -413,6 +407,14 @@ class StatusWidget(QWidget):
         elif status in 'C?':
             qtlib.editfiles(self.repo, [path])
 
+    @pyqtSlot(str)
+    def setStatusFilter(self, status):
+        status = str(status)
+        for s in statusTypes:
+            val = statusTypes[s]
+            self.opts[val.name] = s in status
+        self.refreshWctx()
+
     @pyqtSlot(QString)
     def setFilter(self, match):
         model = self.tv.model()
@@ -420,6 +422,13 @@ class StatusWidget(QWidget):
             model.setFilter(match)
             self.tv.enablefilterpalette(bool(match))
 
+    @pyqtSlot()
+    def clearPattern(self):
+        self.pats = []
+        self.refreshWctx()
+        self.clearPatternBtn.setVisible(False)
+        self.titleTextChanged.emit(self.getTitle())
+
     @pyqtSlot()
     def updateCheckCount(self):
         'user has toggled one or more checkboxes, update counts and checkall'
@@ -1050,12 +1059,6 @@ class StatusDialog(QDialog):
             from tortoisehg.hgqt.run import qtrun
             from tortoisehg.hgqt import commit
             qtrun(commit.run, self.stwidget.repo.ui, root=link[len('repo:'):])
-        if link.startswith('shelve:'):
-            from tortoisehg.hgqt import shelve
-            dlg = shelve.ShelveDialog(self.stwidget.repo, self)
-            dlg.finished.connect(dlg.deleteLater)
-            dlg.exec_()
-            self.refresh()
 
     def loadSettings(self):
         s = QSettings()
@@ -1080,8 +1083,6 @@ class StatusDialog(QDialog):
         QDialog.reject(self)
 
 def run(ui, *pats, **opts):
-    from tortoisehg.util import paths
-    from tortoisehg.hgqt import thgrepo
     repo = thgrepo.repository(ui, path=paths.find_root())
     pats = hglib.canonpaths(pats)
     os.chdir(repo.root)
